@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -58,6 +58,18 @@ def hasRootModule(module_name):
     return False
 
 
+def replaceRootModule(old, new):
+    # Using global here, as this is really a singleton, in the form of a module,
+    # pylint: disable=W0603
+    global root_modules
+    new_root_modules = OrderedSet()
+
+    for module in root_modules:
+        new_root_modules.add(module if module is not old else new)
+
+    root_modules = new_root_modules
+
+
 def addUncompiledModule(module):
     uncompiled_modules.add(module)
 
@@ -69,6 +81,34 @@ def getUncompiledModules():
     )
 
 
+def getUncompiledTechnicalModules():
+    result = [
+        module
+        for module in
+        uncompiled_modules
+        if module.isTechnical()
+    ]
+
+    return sorted(
+        result,
+        key = lambda module : module.getFullName()
+    )
+
+
+def getUncompiledNonTechnicalModules():
+    result = [
+        module
+        for module in
+        uncompiled_modules
+        if not module.isTechnical()
+    ]
+
+    return sorted(
+        result,
+        key = lambda module : module.getFullName()
+    )
+
+
 def _normalizeModuleFilename(filename):
     if python_version >= 300:
         filename = filename.replace("__pycache__", "")
@@ -150,7 +190,6 @@ def getDoneUserModules():
             for module in
             done_modules
             if not module.isInternalModule()
-            if not module.isMainModule()
         ],
         key = lambda module : module.getFullName()
     )
