@@ -20,6 +20,7 @@
 """
 
 import hashlib
+import sys
 
 from nuitka import Options
 from nuitka.__past__ import iterItems
@@ -438,6 +439,11 @@ def _getConstantDefaultPopulation():
             "types"
         )
 
+    if not Options.shallMakeModule():
+        result.append(
+            sys.executable
+        )
+
     return result
 
 
