@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -48,13 +48,40 @@ struct __initResourceConstants
 extern "C" const unsigned char constant_bin[];
 #endif
 
-static void __initConstants( void )
+static void _createGlobalConstants( void )
 {
-%(constant_locals)s
+    NUITKA_MAY_BE_UNUSED PyObject *exception_type, *exception_value;
+    NUITKA_MAY_BE_UNUSED PyTracebackObject *exception_tb;
+
+#ifdef _MSC_VER
+    // Prevent unused warnings in case of simple programs, the attribute
+    // NUITKA_MAY_BE_UNUSED doesn't work for MSVC.
+    (void *)exception_type; (void *)exception_value; (void *)exception_tb;
+#endif
+
 %(constant_inits)s
+
+#if _NUITKA_EXE
+    /* Set the "sys.executable" path to the original CPython executable. */
+    PySys_SetObject(
+        (char *)"executable",
+        %(sys_executable)s
+    );
+#endif
 }
 
-void _initConstants( void )
+// In debug mode we can check that the constants were not tampered with in any
+// given moment. We typically do it at program exit, but we can add extra calls
+// for sanity.
+#ifndef __NUITKA_NO_ASSERT__
+void checkGlobalConstants( void )
+{
+%(constant_checks)s
+}
+#endif
+
+
+void createGlobalConstants( void )
 {
     if ( _sentinel_value == NULL )
     {
@@ -66,14 +93,10 @@ void _initConstants( void )
 #endif
         assert( _sentinel_value );
 
-        __initConstants();
+        _createGlobalConstants();
     }
 }
 """
 
-template_constants_declaration = """\
-// Call this to initialize all of the below
-void _initConstants( void );
-
-%(constant_declarations)s
-"""
+from . import TemplateDebugWrapper # isort:skip
+TemplateDebugWrapper.checkDebug(globals())
