@@ -24,14 +24,16 @@ make others possible.
 
 
 import inspect
-from logging import debug, info, warning
+from logging import debug, info
 
 from nuitka import ModuleRegistry, Options, VariableRegistry
-from nuitka.optimizations import TraceCollections
+from nuitka.importing import ImportCache
+from nuitka.optimizations import Graphs, TraceCollections
 from nuitka.plugins.Plugins import Plugins
 from nuitka.Tracing import printLine
 from nuitka.utils import MemoryUsage
 
+from .BytecodeDemotion import demoteCompiledModuleToBytecode
 from .Tags import TagSet
 
 _progress = Options.isShowProgress()
@@ -69,10 +71,8 @@ def signalChange(tags, source_ref, message):
 # Use this globally from there, without cyclic dependency.
 TraceCollections.signalChange = signalChange
 
-graph = None
-computation_counters = {}
 
-def optimizePythonModule(module):
+def optimizeCompiledPythonModule(module):
     if _progress:
         printLine(
             "Doing module local optimizations for '{module_name}'.".format(
@@ -80,14 +80,9 @@ def optimizePythonModule(module):
             )
         )
 
-    # The tag set is global, so it can react to changes without context.
-    # pylint: disable=W0603
-    global tag_set
-    tag_set = TagSet()
-
     touched = False
 
-    if _progress:
+    if _progress and Options.isShowMemory():
         memory_watch = MemoryUsage.MemoryWatch()
 
     while True:
@@ -99,18 +94,21 @@ def optimizePythonModule(module):
             info("Interrupted while working on '%s'." % module)
             raise
 
-        if not tag_set:
+        Graphs.onModuleOptimizationStep(module)
+
+        # Search for local change tags.
+        for tag in tag_set:
+            if tag == "new_code":
+                continue
+
+            break
+        else:
             break
 
-        if graph is not None:
-            computation_counters[module] = computation_counters.get(module, 0) + 1
-            module_graph = module.asGraph(computation_counters[module])
-
-            graph.subgraph(module_graph)
-
+        # Otherwise we did stuff, so note that for return value.
         touched = True
 
-    if _progress:
+    if _progress and Options.isShowMemory():
         memory_watch.finish()
 
         printLine(
@@ -124,19 +122,45 @@ def optimizePythonModule(module):
 
     return touched
 
+def optimizeUncompiledPythonModule(module):
+    if _progress:
+        printLine(
+            "Doing module dependency considerations for '{module_name}':".format(
+                module_name = module.getFullName()
+            )
+        )
+
+    for used_module_name in module.getUsedModules():
+        used_module = ImportCache.getImportedModuleByName(used_module_name)
+        ModuleRegistry.addUsedModule(used_module)
+
+    package_name = module.getPackage()
+
+    if package_name is not None:
+        used_module = ImportCache.getImportedModuleByName(package_name)
+        ModuleRegistry.addUsedModule(used_module)
+
 
 def optimizeShlibModule(module):
     # Pick up parent package if any.
     _attemptRecursion(module)
 
-    # The tag set is global, so it can react to changes without context.
-    # pylint: disable=W0603
-    global tag_set
-    tag_set = TagSet()
-
     Plugins.considerImplicitImports(module, signal_change = signalChange)
 
 
+def optimizeModule(module):
+    if module.isPythonShlibModule():
+        optimizeShlibModule(module)
+        changed = False
+    elif module.isCompiledPythonModule():
+        changed = optimizeCompiledPythonModule(module)
+    else:
+        optimizeUncompiledPythonModule(module)
+        changed = False
+
+    return changed
+
+
 def areEmptyTraces(variable_traces):
     empty = True
 
@@ -265,88 +289,111 @@ def optimizeUnusedTempVariables(provider):
 
 
 def optimizeVariables(module):
-    for function_body in module.getUsedFunctions():
-        if not VariableRegistry.complete:
-            continue
+    if module.isCompiledPythonModule():
+        for function_body in module.getUsedFunctions():
+            if not VariableRegistry.complete:
+                continue
 
-        optimizeUnusedUserVariables(function_body)
+            optimizeUnusedUserVariables(function_body)
 
-        optimizeUnusedClosureVariables(function_body)
+            optimizeUnusedClosureVariables(function_body)
 
-        optimizeUnusedTempVariables(function_body)
+            optimizeUnusedTempVariables(function_body)
 
-    optimizeUnusedTempVariables(module)
+        optimizeUnusedTempVariables(module)
+
+
+def _traceProgress(current_module):
+    output = """\
+Optimizing module '{module_name}', {remaining:d} more modules to go \
+after that.""".format(
+            module_name = current_module.getFullName(),
+            remaining   = ModuleRegistry.remainingCount(),
+    )
+
+    if Options.isShowMemory():
+        output += "Memory usage {memory}:".format(
+            memory = MemoryUsage.getHumanReadableProcessMemoryUsage()
+        )
+
+    printLine(output)
+
+
+def makeOptimizationPass(initial_pass):
+    """ Make a single pass for optimization, indication potential completion.
+
+    """
+    finished = True
+
+    ModuleRegistry.startTraversal()
+
+    if _progress:
+        if initial_pass:
+            printLine("Initial optimization pass.")
+        else:
+            printLine("Next global optimization pass.")
+
+    while True:
+        current_module = ModuleRegistry.nextModule()
+
+        if current_module is None:
+            break
+
+        if _progress:
+            _traceProgress(current_module)
+
+        # The tag set is global, so it can react to changes without context.
+        # pylint: disable=W0603
+        global tag_set
+        tag_set = TagSet()
+
+        changed = optimizeModule(current_module)
+
+        if changed:
+            finished = False
+
+    # Unregister collection traces from now unused code, dropping the trace
+    # collections of functions no longer used.
+    for current_module in ModuleRegistry.getDoneModules():
+        if current_module.isCompiledPythonModule():
+            for function in current_module.getUnusedFunctions():
+                VariableRegistry.updateFromCollection(
+                    old_collection = function.constraint_collection,
+                    new_collection = None
+                )
+
+                function.constraint_collection = None
+
+    for current_module in ModuleRegistry.getDoneModules():
+        optimizeVariables(current_module)
+
+    return finished
 
 
 def optimize():
-    # This is somewhat complex with many cases, pylint: disable=R0912
+    Graphs.startGraph()
 
-    # We maintain this globally to make it accessible, pylint: disable=W0603
-    global graph
+    # First pass.
+    if _progress:
+        info("PASS 1:")
 
-    if Options.shouldCreateGraph():
+    makeOptimizationPass(False)
+    VariableRegistry.considerCompletion()
+    finished = makeOptimizationPass(False)
 
-        try:
-            from graphviz import Digraph # pylint: disable=F0401,I0021
-            graph = Digraph('G')
-        except ImportError:
-            warning("Cannot import graphviz module, no graphing capability.")
+    # Demote to bytecode if now.
+    for module in ModuleRegistry.getDoneUserModules():
+        if module.isPythonShlibModule():
+            continue
 
-    while True:
-        finished = True
+        if module.mode == "bytecode":
+            demoteCompiledModuleToBytecode(module)
 
-        ModuleRegistry.startTraversal()
+    # Second, endless pass.
+    if _progress:
+        info("PASS 2..:")
 
-        while True:
-            current_module = ModuleRegistry.nextModule()
+    while not finished:
+        finished = makeOptimizationPass(True)
 
-            if current_module is None:
-                break
-
-            if _progress:
-                printLine(
-                    """\
-Optimizing module '{module_name}', {remaining:d} more modules to go \
-after that. Memory usage {memory}:""".format(
-                        module_name = current_module.getFullName(),
-                        remaining   = ModuleRegistry.remainingCount(),
-                        memory      = MemoryUsage.getHumanReadableProcessMemoryUsage()
-                    )
-                )
-
-            if current_module.isPythonShlibModule():
-                optimizeShlibModule(current_module)
-            else:
-                changed = optimizePythonModule(current_module)
-
-                if changed:
-                    finished = False
-
-        # Unregister collection traces from now unused code.
-        for current_module in ModuleRegistry.getDoneModules():
-            if not current_module.isPythonShlibModule():
-                for function in current_module.getUnusedFunctions():
-                    VariableRegistry.updateFromCollection(
-                        old_collection = function.constraint_collection,
-                        new_collection = None
-                    )
-
-                    function.constraint_collection = None
-
-        if VariableRegistry.considerCompletion():
-            finished = False
-
-        for current_module in ModuleRegistry.getDoneModules():
-            if not current_module.isPythonShlibModule():
-                optimizeVariables(current_module)
-
-        if finished:
-            break
-
-
-    if graph is not None:
-        graph.engine = "dot"
-        graph.graph_attr["rankdir"] = "TB"
-        graph.render("something.dot")
-
-        printLine(graph.source)
+    Graphs.endGraph()
