@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,9 +27,12 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
-from nuitka.nodes.SliceNodes import ExpressionSliceLookup, ExpressionSliceObject
+from nuitka.nodes.SliceNodes import (
+    ExpressionBuiltinSlice,
+    ExpressionSliceLookup
+)
 from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
-from nuitka.utils import Utils
+from nuitka.PythonVersions import python_version
 
 from .Helpers import buildNode, getKind
 from .ReformulationAssignmentStatements import buildExtSliceNode
@@ -80,14 +83,14 @@ def buildSubscriptNode(provider, node, source_ref):
         # For Python3 there is no slicing operation, this is always done
         # with subscript using a slice object. For Python2, it is only done
         # if no "step" is provided.
-        use_sliceobj = step is not None or Utils.python_version >= 300
+        use_sliceobj = step is not None or python_version >= 300
 
         if use_sliceobj:
             return ExpressionSubscriptLookup(
                 subscribed = buildNode(provider, node.value, source_ref),
-                subscript  = ExpressionSliceObject(
-                    lower      = lower,
-                    upper      = upper,
+                subscript  = ExpressionBuiltinSlice(
+                    start      = lower,
+                    stop       = upper,
                     step       = step,
                     source_ref = source_ref
                 ),
